/**
 * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 */

require('@open-xchange/appsuite-codeceptjs/src/chai')
const dotenv = require('dotenv')
dotenv.config({ path: '.env' })
dotenv.config({ path: '.env.defaults' })

const requiredEnvVars = ['LAUNCH_URL', 'PROVISIONING_URL']

requiredEnvVars.forEach(function notdefined (key) {
  if (process.env[key]) return
  console.error('\x1b[31m', `ERROR: Missing value for environment variable '${key}'. Please specify a '.env' file analog to '.env-example'.`)
  process.exit()
})

module.exports = {
  util: require('./src/util'),
  event: require('./src/event'),
  config: {
    tests: './tests/**/*_test.js',
    timeout: Number(process.env.TEST_TIMEOUT) || 120,
    output: './output/',
    helpers: {
      Playwright: {
        timeout: 10000,
        waitForTimeout: 10000,
        browser: 'chromium',
        chromium: {
          args: [
            '--disable-print-preview',
            '--disable-crash-reporter',
            '--disable-dev-shm-usage',
            '--disable-features=IsolateOrigins',
            '--disable-gpu',
            '--disable-notifications', // to disable native notification window on Mac OS,
            '--disable-print-preview',
            '--disable-setuid-sandbox',
            '--disable-site-isolation-trials',
            '--disable-web-security',
            '--no-first-run',
            '--no-sandbox',
            '--no-zygote'
          ].concat((process.env.CHROME_ARGS || '').split(' '))
        },
        url: process.env.LAUNCH_URL,
        show: process.env.HEADLESS === 'false',
        trace: process.env.TRACE === 'true',
        restart: true // this is needed to be able to switch browsers in tests
      },
      AppSuite: {
        require: '@open-xchange/appsuite-codeceptjs/src/helper',
        mxDomain: process.env.MX_DOMAIN,
        serverURL: process.env.PROVISIONING_URL,
        contextId: process.env.CONTEXT_ID,
        filestoreId: process.env.FILESTORE_ID,
        smtpServer: process.env.SMTP_SERVER,
        imapServer: process.env.IMAP_SERVER,
        loginTimeout: 30,
        admin: {
          login: process.env.E2E_ADMIN_USER,
          password: process.env.E2E_ADMIN_PW
        }
      }
    },
    include: {
      I: '@open-xchange/appsuite-codeceptjs/src/actor',
      users: '@open-xchange/appsuite-codeceptjs/src/users',
      contexts: '@open-xchange/appsuite-codeceptjs/src/contexts',
      // pageobjects
      contacts: '@open-xchange/appsuite-codeceptjs/src/pageobjects/contacts',
      calendar: '@open-xchange/appsuite-codeceptjs/src/pageobjects/calendar',
      mail: '@open-xchange/appsuite-codeceptjs/src/pageobjects/mail',
      drive: '@open-xchange/appsuite-codeceptjs/src/pageobjects/drive',
      tasks: '@open-xchange/appsuite-codeceptjs/src/pageobjects/tasks',
      // fragments
      dialogs: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/dialogs',
      autocomplete: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/contact-autocomplete',
      contactpicker: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/contact-picker',
      mailfilter: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/settings-mailfilter',
      search: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/search',
      tinymce: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/tinymce',
      topbar: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/topbar',
      settings: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/settings',
      viewer: '@open-xchange/appsuite-codeceptjs/src/pageobjects/fragments/viewer',
      mobileCalendar: '@open-xchange/appsuite-codeceptjs/src/pageobjects/mobile/mobileCalendar',
      mobileMail: '@open-xchange/appsuite-codeceptjs/src/pageobjects/mobile/mobileMail',
      mobileContacts: '@open-xchange/appsuite-codeceptjs/src/pageobjects/mobile/mobileContacts'
    },
    async bootstrap () {
      // set moment defaults
      // note: no need to require moment-timezone later on. requiring moment is enough
      const moment = require('moment')
      require('moment-timezone')
      moment.tz.setDefault('Europe/Berlin')

      const codecept = require('codeceptjs')

      const config = codecept.config.get()
      const helperConfig = config.helpers.AppSuite

      const contexts = codecept.container.support('contexts')

      const testRunContext = await contexts.create()
      if (typeof testRunContext.id !== 'undefined') helperConfig.contextId = testRunContext.id
    },
    async teardown () {
      const { contexts } = global.inject()
      // we need to run this sequentially, less stress on the MW
      for (const ctx of contexts.filter(ctx => /e2e-context-/.test(String(ctx.loginMappings)))) {
        await ctx.remove().catch(e => console.error(e.message))
      }
    },
    reporter: 'mocha-multi',
    mocha: {
      reporterOptions: {
        'codeceptjs-cli-reporter': {
          stdout: '-'
        },
        'mocha-junit-reporter': {
          stdout: '-',
          options: {
            jenkinsMode: true,
            mochaFile: './output/junit.xml',
            attachments: false // add screenshot for a failed test
          }
        }
      }
    },
    plugins: {
      allure: {
        enabled: true,
        require: 'allure-codeceptjs',
        outputDir: './output'
        // This will change to the below once the allure plugin is updated to >= 3.0.0
        // At the time of writing, the allure plugin is at 3.0.0-beta.3 and screenshots are not working
        // resultsDir: './output'
      },
      testMetrics: {
        require: '@open-xchange/appsuite-codeceptjs/src/plugins/testmetrics',
        url: 'https://e2e-metrics.dev.oxui.de',
        org: 'e2e',
        defaultTags: {
          client: process.env.E2E_TEST_METRICS_CLIENT || (process.env.CI && 'CI'),
          pipelineId: process.env.CI_PIPELINE_ID,
          branch: process.env.CI_COMMIT_REF_NAME,
          project: process.env.CI_PROJECT_PATH
        },
        token: process.env.E2E_TEST_METRICS_TOKEN,
        enabled: true
      },
      settingsInit: {
        require: '@open-xchange/appsuite-codeceptjs/src/plugins/settingsInit',
        enabled: true
      },
      customizePlugin: {
        require: process.env.CUSTOMIZE_PLUGIN || '@open-xchange/appsuite-codeceptjs/src/plugins/emptyModule',
        enabled: true
      },
      filterSuite: {
        enabled: !process.env.E2E_GREP && process.env.CI && (process.env.SUITE_SERVICE_URL || process.env.FILTER_SUITE),
        require: '@open-xchange/codecept-horizontal-scaler',
        suiteFilePath: process.env.FILTER_SUITE,
        filterFn: process.env.runOnly === 'true' ? () => false : undefined,
        nodePrefix: 'core-ui'
      }
    },
    rerun: {
      minSuccess: Number(process.env.MIN_SUCCESS),
      maxReruns: Number(process.env.MAX_RERUNS)
    }
  }
}
