/**
 * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 */

const event = require('../event')
const users = require('../users/reseller')()
const util = require('../util')
const resellerContextService = require('../soap/services/resellerContext')
const resellerUserService = require('../soap/services/resellerUser')
const contexts = []
const created = []

class ResellerContext {
  constructor (ctx, admin, auth) {
    this.ctxdata = ctx
    this.admin = admin
    this.auth = auth
    this.id = ctx.id
    this.userAttributes = ctx.userAttributes

    return new Proxy(this, {
      get (target, prop) {
        return prop in target ? target[prop] : target.ctxdata[prop]
      }
    })
  }

  static async removeAll (auth) {
    let ctx = created.pop()
    while (ctx) {
      if (auth) ctx.auth = auth
      await ctx.remove()
      ctx = created.pop()
    }
    return ResellerContext
  }

  async remove () {
    await resellerContextService.remove(this.id)
    contexts.splice(0, contexts.length, ...contexts.filter(c => c !== this))
    created.splice(0, created.length, ...created.filter(c => c !== this))
    users.splice(0, users.length, ...users.filter(u => u.context.id !== this.id))
    event.emit(event.provisioning.context.removed, this)
  }

  hasCapability (capability) {
    return this._setCapability(capability, 'true')
  }

  _setCapability (cap, value) {
    const userAttributes = this.userAttributes || { entries: [] }
    const configMap = userAttributes.entries.find(entry => entry.key === 'config') || { key: 'config' }
    configMap.value = configMap.value || { entries: [] }
    configMap.value.entries.push({ key: `com.openexchange.capability.${cap}`, value })
    return resellerContextService
      .change({ id: this.ctxdata.id, userAttributes })
      .then(() => resellerContextService.get(this.ctxdata.id))
  }

  doesntHaveCapability (capability) {
    return this._setCapability(capability, false)
  }

  hasConfig (key, value) {
    // Config structure is the following { userAttributes: { entries: [{ key: 'config', value: { entries: [{ key: 'key', value: value }] }}]} }
    // Note that the soap lib will replace arrays with a single element by that element

    // check if config exists and create config path if it does not exist
    const emptyConfig = { entries: [{ key: 'config', value: null }] }
    this.ctxdata.userAttributes = this.ctxdata.userAttributes || emptyConfig
    const userAttributes = this.ctxdata.userAttributes
    const entries = userAttributes.entries = userAttributes.entries ? [].concat(userAttributes.entries) : []
    let configEntry = entries.find(e => e.key === 'config')
    if (!configEntry) {
      entries.push({ key: 'config', value: {} })
      configEntry = entries[entries.length - 1]
    }
    const config = configEntry.value = configEntry.value || { entries: [] }
    const configEntries = config.entries = config.entries ? [].concat(config.entries) : []
    let targetConfig = configEntries.find(e => e.key === key)
    if (!targetConfig) {
      configEntries.push({ key })
      targetConfig = configEntries[configEntries.length - 1]
    }
    targetConfig.value = value

    return resellerContextService.change({ id: this.ctxdata.id, userAttributes })
  }

  hasTaxonomy (taxonomy) {
    // check if config exists and create config path if it does not exist
    const userAttributes = this.ctxdata.userAttributes || {}
    const entries = userAttributes.entries = userAttributes.entries ? [].concat(userAttributes.entries) : []
    let taxonomyEntry = entries.find(e => e.key === 'taxonomy')
    if (!taxonomyEntry) {
      entries.push({ key: 'taxonomy', value: { entries: [{ key: 'types' }] } })
      taxonomyEntry = entries[entries.length - 1]
    }
    const targetEntry = taxonomyEntry.value.entries.find(e => e.key === 'types')
    targetEntry.value = taxonomy

    return resellerContextService.change({ id: this.ctxdata.id, userAttributes })
  }

  hasQuota (maxQuota) {
    this.ctxdata.maxQuota = maxQuota
    return resellerContextService.change({ id: this.id, maxQuota })
  }

  getModuleAccess () {
    return resellerContextService.getModuleAccess(this.id)
  }

  async hasModuleAccess (moduleAccess) {
    return resellerContextService.changeModuleAccess(this.id, moduleAccess)
  }

  get users () {
    return resellerUserService.listAll({
      ctx: { id: this.ctxdata.id }
    }).then(([{ return: list }]) => {
      return Promise.all(
        list.map(u =>
          resellerUserService.get({
            user: { id: u.id },
            ctx: { id: this.ctxdata.id },
            auth: this.auth
          }).then(([{ return: user }]) => user, (err) => { throw new util.PropagatedError(err) })
        )
      )
    }, (err) => { throw new util.PropagatedError(err) })
  }

  static defaultAdmin () {
    return {
      password: 'secret',
      display_name: 'context admin',
      sur_name: 'admin',
      given_name: 'context',
      email1: `oxadmin@${util.mxDomain()}`,
      primaryEmail: `oxadmin@${util.mxDomain()}`
    }
  }

  static async create (ctx = {}, adminUser = {}, auth = util.admin(), numberOfUsers = Number(process.env.PROVISIONING_USERS || 10)) {
    ctx = Object.assign({ id: util.userContextId(), maxQuota: -1 }, ctx)
    adminUser = Object.assign({
      name: `${ctx.name || ctx.id}_admin`
    }, this.defaultAdmin(), adminUser)
    let data; let neededPrefix = ctx._auto_prefix
    delete ctx._auto_prefix
    ctx.name = (process.env.CONTEXT_PREFIX || '') + (ctx.name || `${ctx.id}`)
    event.emit(event.provisioning.context.create, ctx, adminUser, auth)
    try {
      data = await resellerContextService.create({
        ctx,
        admin_user: adminUser,
        auth
      })
    } catch (error) {
      if (!neededPrefix && error.message.match(/name must beginn with ([^ ;]+)/)) {
        const oldName = ctx.name
        delete ctx.name
        neededPrefix = error.message.match(/name must beginn with ([^ ;]+)/)[1]
        return ResellerContext.create(Object.assign({
          _auto_prefix: true,
          name: `${neededPrefix}${oldName}`
        }, ctx), adminUser, auth)
      }
      if (error.message.match(/already exists/)) {
        const id = String(util.addJitter(Number.parseInt(ctx.id, 10)))
        return ResellerContext.create(Object.assign({}, ctx, { name: ctx.name.replace(/_\d+/, `_${id}`), id }))
      }
      throw new util.PropagatedError(error)
    }
    const index = created.push(new ResellerContext(data, adminUser, auth)) - 1
    event.emit(event.provisioning.context.created, created[index])

    for (let i = 0; i < numberOfUsers; i++) {
      // @ts-ignore
      await users.create(users.getRandom(), created[index], true)
    }

    return created[index]
  }

  static async reuse (ctx, adminUser = ResellerContext.defaultAdmin(), auth = util.admin()) {
    const existingContext = created.find(c => (c.name === ctx.name) || (c.id === ctx.id) || (new RegExp(`_${ctx.id}$`).test(c.name)))
    if (existingContext) return existingContext

    await ResellerContext.fetchContexts()
    const remoteCtx = contexts.find(c => (c.name === ctx.name) || (c.id === ctx.id) || (new RegExp(`_${ctx.id}$`).test(c.name)))
    if (!remoteCtx.auth) remoteCtx.auth = auth
    if (!remoteCtx.admin_user) remoteCtx.admin_user = adminUser
    return remoteCtx
  }

  static async all () {
    if (contexts.length === 0) await this.fetchContexts()
    return created.concat(contexts)
  }

  static async fetchContexts () {
    const list = await resellerContextService.listAll()
      .catch((err) => { throw new util.PropagatedError(err) })
    contexts.splice(0, contexts.length)
    contexts.push.apply(contexts, list.map(c => new ResellerContext(c)))
  }
}

// can't return the class directly, because codecept tries to execute functions
module.exports = function () {
  ResellerContext.fetchContexts()
  return new Proxy(created, {
    // act like an array if an index is being used
    get (target, prop) {
      if (prop in target) return target[prop]
      return ResellerContext[prop]
    }
  })
}
